@@ -4,31 +4,35 @@
     Módulo para trabalhar com imagens
 """
 from __future__ import annotations
+
+import os.path
 from typing import Union
 from abc import ABC, abstractmethod
-from hashlib import md5 as md5_hash
 from soup_files import File, Directory, InputFiles, LibraryDocs, ProgressBarAdapter
 from io import BytesIO
 import numpy as np
 from scipy.ndimage import uniform_filter
-from convert_stream.enum_libs.enums import LibImage, RotationAngle
-from convert_stream.enum_libs.modules import (
+from sheet_stream.type_utils import get_hash_from_bytes, MetaDataFile, MetaDataItem
+from convert_stream.mod_types.enums import LibImage, RotationAngle
+from convert_stream.mod_types.table_types import ListItems
+from convert_stream.mod_types.modules import (
     MOD_IMG_PIL, ModuleImage, DEFAULT_LIB_IMAGE,
-    cv2, MatLike, Image, ImageFilter
+    cv2, MatLike, Image, ImageFilter, LibImage
 )
 
 
-def get_hash_from_bytes(bt: BytesIO) -> str:
-    return md5_hash(bt.getvalue()).hexdigest().upper()
-
-
 class ABCImageObject(ABC):
 
-    def __init__(self, module_image: ModuleImage, name: str) -> None:
-        self.max_size: tuple[int, int] = (1980, 720) # Dimensões máximas, altere se necessário.
+    def __init__(self, module_image: ModuleImage) -> None:
+        # Dimensões máximas, altere se necessário.
+        self.max_size: tuple[int, int] = (1980, 720)
         self.module_image: ModuleImage = module_image
         self.lib_image: LibImage = LibImage.OPENCV
-        self.name: str = name
+        self.metadata: MetaDataFile = MetaDataFile()
+
+    @property
+    def name(self) -> str | None:
+        return self.metadata.name if not self.metadata.name.is_empty else None
 
     @abstractmethod
     def get_dimensions(self) -> tuple[int, int]:
@@ -89,8 +93,8 @@ class ABCImageObject(ABC):
 
 
 class ImplementOpenCv(ABCImageObject):
-    def __init__(self, module_image: ModuleImage, name: str) -> None:
-        super().__init__(module_image, name)
+    def __init__(self, module_image: ModuleImage) -> None:
+        super().__init__(module_image)
         self.lib_image = LibImage.OPENCV
 
         # Redimensionar se necessário
@@ -202,7 +206,9 @@ class ImplementOpenCv(ABCImageObject):
         img = cv2.imread(f.absolute())
         if img is None:
             raise ValueError(f"Não foi possível abrir a imagem: {f.absolute()}")
-        return cls(img, f.name())
+        im_obj = cls(img)
+        im_obj.metadata = MetaDataFile.create_metadata(f)
+        return im_obj
 
     @classmethod
     def create_from_bytes(cls, bt: bytes) -> ABCImageObject:
@@ -211,14 +217,17 @@ class ImplementOpenCv(ABCImageObject):
         if img is None:
             raise ValueError("Não foi possível criar a imagem a partir dos bytes fornecidos.")
         bytes_io = BytesIO(bt)
-        name = get_hash_from_bytes(bytes_io)
+        name: str = get_hash_from_bytes(bytes_io)
         bytes_io.close()
-        return cls(img, name)
+        im_obj = cls(img)
+        im_obj.metadata.name = MetaDataItem(name)
+        im_obj.metadata.md5 = MetaDataItem(name)
+        return im_obj
 
 
 class ImplementPIL(ABCImageObject):
-    def __init__(self, module_image: ModuleImage, name: str):
-        super().__init__(module_image, name)
+    def __init__(self, module_image: ModuleImage):
+        super().__init__(module_image)
         self.lib_image = LibImage.PIL
 
         # Redimensionar, se as dimensões estiverem maior que self.max_size.
@@ -341,8 +350,10 @@ class ImplementPIL(ABCImageObject):
 
     @classmethod
     def create_from_file(cls, f: File) -> ABCImageObject:
-        img = Image.open(f.path)
-        return cls(img, f.name())
+        img = Image.open(f.absolute())
+        im_obj = cls(img)
+        im_obj.metadata = MetaDataFile.create_metadata(f)
+        return im_obj
 
     @classmethod
     def create_from_bytes(cls, bt: bytes) -> ABCImageObject:
@@ -350,7 +361,10 @@ class ImplementPIL(ABCImageObject):
         bytes_io = BytesIO(bt)
         name = get_hash_from_bytes(bytes_io)
         bytes_io.close()
-        return cls(img, name)
+        _obj = cls(img)
+        _obj.metadata.name = MetaDataItem(name)
+        _obj.metadata.md5 = MetaDataItem(name)
+        return _obj
 
 
 class ImageObject(object):
@@ -398,6 +412,14 @@ class ImageObject(object):
     def name(self) -> str:
         return self.img_adapter.name
 
+    @property
+    def metadata(self) -> MetaDataFile:
+        return self.img_adapter.metadata
+
+    @metadata.setter
+    def metadata(self, metadata: MetaDataFile) -> None:
+        self.img_adapter.metadata = metadata
+
     def to_pil(self) -> Image.Image:
         """
         Converte o ImageObject para um objeto PIL.Image.
@@ -473,44 +495,36 @@ class ImageObject(object):
 
     @classmethod
     def create_from_pil(cls, pil: ModuleImage) -> ImageObject:
-        name = get_hash_from_bytes(BytesIO(pil.tobytes()))
-        return cls(ImplementPIL(pil, name))
+        return cls(ImplementPIL(pil))
 
 
-class CollectionImages(object):
+class CollectionImages(ListItems):
     
-    def __init__(self, images: list[ImageObject] = []) -> None:
+    def __init__(self, images: list[ImageObject], *, lib_img: LibImage = LibImage.OPENCV) -> None:
         """
             Gerir uma lista de Imagens
         :type images: list[ImageObject]
         """
-        self.images: list[ImageObject] = images
+        super().__init__(images)
+        self.lib_img: LibImage = lib_img
         self.pbar: ProgressBarAdapter = ProgressBarAdapter()
         self.__count: int = 0
 
-    @property
-    def length(self) -> int:
-        return len(self.images)
-
-    @property
-    def is_empty(self) -> bool:
-        return len(self.images) == 0
-
     def clear(self) -> None:
-        self.images.clear()
+        super().clear()
         self.__count = 0
 
     def add_image(self, image: ImageObject) -> None:
         self.__count += 1
         self.pbar.update_text(f'Adicionando imagem {self.__count}')
-        self.images.append(image)
+        self.append(image)
 
     def add_file_image(self, file: File) -> None:
         self.pbar.start()
         self.__count += 1
         self.pbar.update_text(f'Adicionando imagem {self.__count} {file.basename()}')
-        im = ImageObject.create_from_file(file)
-        self.images.append(im)
+        im = ImageObject.create_from_file(file, lib_image=self.lib_img)
+        self.append(im)
         self.pbar.stop()
 
     def add_images(self, images: list[ImageObject]) -> None:
@@ -525,7 +539,7 @@ class CollectionImages(object):
                 ((num+1) / max_num) * 100,
                 f'Adicionando imagem: [{num+1} de {max_num}]'
             )
-            self.images.append(ImageObject.create_from_file(f))
+            self.append(ImageObject.create_from_file(f, lib_image=self.lib_img))
         self.pbar.stop()
 
     def add_directory_images(self, d: Directory, max_files: int = 4000) -> None:
@@ -542,9 +556,13 @@ class CollectionImages(object):
         self.pbar.start()
         print()
         output_dir.mkdir()
-        max_num = len(self.images)
-        for n, img in enumerate(self.images):
-            filename = f'{img.name}.png'
+        max_num = self.length
+        img: ImageObject
+        for n, img in enumerate(self):
+            if img.metadata.file_path is not None:
+                filename = f'{os.path.basename(img.metadata.file_path)}-{n}.png'
+            else:
+                filename = f'{img.metadata.name}-{n}.png'
             file_path = output_dir.join_file(filename)
             if (not replace) and (file_path.exists()):
                 self.pbar.update_text(f'[PULANDO]: {file_path.basename()}')
@@ -566,13 +584,13 @@ class CollectionImages(object):
         self.pbar.stop()
 
     def set_land_scape(self):
-        for im in self.images:
+        for im in self:
             im.set_landscape()
 
     def set_gausian_blur(self, sigma: float = 0.8) -> None:
         self.pbar.start()
         max_num: int = self.length
-        for _num, im in enumerate(self.images):
+        for _num, im in enumerate(self):
             self.pbar.update(
                 ((_num + 1) / max_num) * 100,
                 f'Aplicando GausianBlur: [{_num + 1} de {max_num}]'
@@ -585,3 +603,72 @@ class CollectionImages(object):
     def set_pbar(self, p: ProgressBarAdapter) -> None:
         self.pbar = p
 
+
+class ProcessImageScanner(object):
+
+    def __init__(self, image: ImageObject):
+        self.image: ImageObject = image
+
+    def select_doc(self) -> ImageObject | None:
+        """
+        Detecta e recorta automaticamente a área do documento principal
+        (texto ou folha digitalizada) em uma imagem.
+        Retorna um novo ImageObject contendo apenas o recorte detectado.
+        """
+        # Converte para matriz OpenCV
+        img_cv: np.ndarray = cv2.imdecode(self.image.to_numpy(), cv2.IMREAD_COLOR)
+        if img_cv is None:
+            print(f'[NDARRAY] Falha na imagem: {self.image.metadata.name}')
+            return None
+
+        # Etapa 1: converter para tons de cinza
+        gray = cv2.cvtColor(img_cv, cv2.COLOR_BGR2GRAY)
+
+        # Etapa 2: aplicar blur para suavizar ruídos
+        gray_blur = cv2.GaussianBlur(gray, (5, 5), 0)
+
+        # Etapa 3: detecção de bordas
+        edges = cv2.Canny(gray_blur, 50, 150)
+
+        # Etapa 4: encontrar contornos
+        contours, _ = cv2.findContours(edges, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)
+        if not contours:
+            print(f'[CONTOURS] Falha na imagem: {self.image.metadata.name}')
+            return None
+
+        # Etapa 5: selecionar o maior contorno plausível (maior área retangular)
+        contours = sorted(contours, key=cv2.contourArea, reverse=True)
+        doc_contour = None
+
+        for c in contours:
+            perimeter = cv2.arcLength(c, True)
+            approx = cv2.approxPolyDP(c, 0.02 * perimeter, True)
+            if len(approx) == 4:
+                doc_contour = approx
+                break
+
+        if doc_contour is None:
+            # Caso não encontre um quadrilátero, usa o maior contorno
+            doc_contour = contours[0]
+
+        # Etapa 6: criar bounding box e recortar
+        x, y, w, h = cv2.boundingRect(doc_contour)
+        cropped = img_cv[y:y + h, x:x + w]
+
+        # Evitar recortes muito pequenos (ruído)
+        if w < 50 or h < 50:
+            return None
+
+        # Etapa 7: converter o recorte em ImageObject
+        success, buffer = cv2.imencode(".png", cropped)
+        if not success:
+            print(f'Falha na imagem: {self.image.metadata.name}')
+            return None
+
+        bytes_io = BytesIO(buffer.tobytes())
+        new_image = ImageObject(bytes_io, lib_image=LibImage.OPENCV)
+        # Herdar metadados originais
+        new_image.metadata = self.image.metadata
+        new_image.metadata.name = MetaDataItem(f"{self.image.metadata.name}_crop")
+        return new_image
+
