@@ -9,87 +9,97 @@ from io import BytesIO
 from typing import Union
 import pandas as pd
 from soup_files import File, Directory, LibraryDocs, InputFiles, ProgressBarAdapter, JsonConvert
-from convert_stream.enum_libs.enums import LibPDF, ColumnsTable
-from convert_stream.enum_libs.modules import (
-    DEFAULT_LIB_PDF, ModDocPdf, PdfWriter, PdfReader, PageObject, fitz,
+from sheet_stream.type_utils import (
+    get_hash_from_bytes, MetaDataFile, MetaDataItem
+)
+
+from sheet_stream.type_utils import (
+    ListItems, HeadCell, HeadValues, ColumnsTable, concat_table_documents, TableDocuments
+)
+from convert_stream.mod_types.table_types import DictTextTable, ColumnBody
+from convert_stream.mod_types.enums import LibPDF
+from convert_stream.mod_types.modules import (
+    DEFAULT_LIB_PDF, ModDocPdf, PdfWriter, PdfReader, fitz, MOD_FITZ, MOD_PYPDF
 )
-from convert_stream.image.img_object import get_hash_from_bytes
 from convert_stream.pdf.pdf_page import PageDocumentPdf
-from convert_stream.text import print_line, show_warning, MapText, FindText
-from convert_stream.sheets.save import save_data
+from convert_stream.text import show_warning, FindText
 
 
 class SearchableTextPdf(object):
     """
-        Esta classe serve para armazenar textos filtrados em documentos PDF
-    guardando o texto buscado e outros elementos como a página em que o texto foi
-    encontrado, linha e o nome do respectivo arquivo.
+
+    create_void_map() -> dict[str, ColumnBody]:
+
+    return {
+            ColumnsTable.KEY.value: ColumnBody(ColumnsTable.KEY.value, ListString([])),
+            ColumnsTable.NUM_PAGE.value: ColumnBody(ColumnsTable.NUM_PAGE.value, ListString([])),
+            ColumnsTable.NUM_LINE.value: ColumnBody(ColumnsTable.NUM_LINE.value, ListString([])),
+            ColumnsTable.TEXT: ColumnBody(ColumnsTable.TEXT, ListString([])),
+            ColumnsTable.FILE_NAME.value: ColumnBody(ColumnsTable.FILE_NAME.value, ListString([])),
+            ColumnsTable.FILETYPE.value: ColumnBody(ColumnsTable.FILETYPE.value, ListString([])),
+            ColumnsTable.FILE_PATH.value: ColumnBody(ColumnsTable.FILE_PATH.value, ListString([])),
+            ColumnsTable.DIR.value: ColumnBody(ColumnsTable.DIR.value, ListString([])),
+        }
     """
     def __init__(self, silent: bool = False):
         self.silent = silent
-        # Cada elemento encontrado na busca de texto, será guardado nesse objeto para
-        # posterior consulta (self.elements), cada chave aqui deve ter uma lista de
-        # tamanho igual.
-        self.elements: dict[str, list[str]] = {
-            ColumnsTable.NUM_PAGE.value: [],
-            ColumnsTable.NUM_LINHA.value: [],
-            ColumnsTable.TEXT.value: [],
-            ColumnsTable.FILE.value: [],
-        }
+        self.elements: dict[str, ColumnBody] = DictTextTable.create_void_dict()
 
     def __repr__(self):
         return f'SearchableTextPdf: {self.elements}'
 
+    @property
+    def length(self) -> int:
+        return len(self.elements[ColumnsTable.TEXT])
+
     def is_empty(self) -> bool:
-        return len(self.elements[ColumnsTable.TEXT.value]) == 0
+        return len(self.elements[ColumnsTable.TEXT]) == 0
+
+    @property
+    def columns(self) -> HeadValues:
+        return HeadValues([HeadCell(x) for x in list(self.elements.keys())])
 
     @property
     def first(self) -> dict[str, str]:
         if self.is_empty():
             return {}
-        return {
-            ColumnsTable.NUM_PAGE.value: self.elements[ColumnsTable.NUM_PAGE.value][0],
-            ColumnsTable.NUM_LINHA.value: self.elements[ColumnsTable.NUM_LINHA.value][0],
-            ColumnsTable.TEXT.value: self.elements[ColumnsTable.TEXT.value][0],
-            ColumnsTable.FILE.value: self.elements[ColumnsTable.FILE.value][0],
-        }
+        cols = self.columns
+        _first = {}
+        for c in cols:
+            _first[c] = self.elements[c][0]
+        return _first
 
     @property
     def last(self) -> dict[str, str]:
         if self.is_empty():
             return {}
-        return {
-            ColumnsTable.NUM_PAGE.value: self.elements[ColumnsTable.NUM_PAGE.value][-1],
-            ColumnsTable.NUM_LINHA.value: self.elements[ColumnsTable.NUM_LINHA.value][-1],
-            ColumnsTable.TEXT.value: self.elements[ColumnsTable.TEXT.value][-1],
-            ColumnsTable.FILE.value: self.elements[ColumnsTable.FILE.value][-1],
-        }
-
-    @property
-    def length(self) -> int:
-        return len(self.elements[ColumnsTable.TEXT.value])
+        if self.is_empty():
+            return {}
+        cols = self.columns
+        _last = {}
+        for c in cols:
+            _last[c] = self.elements[c][-1]
+        return _last
 
     def get_item(self, idx: int) -> dict[str, str]:
+        _current_item = {}
+        cols = self.columns
         try:
-            return {
-                ColumnsTable.NUM_PAGE.value: self.elements[ColumnsTable.NUM_PAGE.value][idx],
-                ColumnsTable.NUM_LINHA.value: self.elements[ColumnsTable.NUM_LINHA.value][idx],
-                ColumnsTable.TEXT.value: self.elements[ColumnsTable.TEXT.value][idx],
-                ColumnsTable.FILE.value: self.elements[ColumnsTable.FILE.value][idx],
-            }
+            for col in cols:
+                _current_item[col] = self.elements[col][idx]
         except Exception as err:
             if not self.silent:
                 print(err)
-            return {}
+            return _current_item
+        else:
+            return _current_item
 
-    def add_line(self, text, *, num_page: str, num_line: str, file: str) -> None:
-        if not self.silent:
-            print_line()
-            print(f'{file} => {text}')
-        self.elements[ColumnsTable.NUM_PAGE.value].append(num_page)
-        self.elements[ColumnsTable.NUM_LINHA.value].append(num_line)
-        self.elements[ColumnsTable.TEXT.value].append(text)
-        self.elements[ColumnsTable.FILE.value].append(file)
+    def add_line(self, line: dict[str, str]) -> None:
+        cols: HeadValues = self.columns
+        cols_line: HeadValues = HeadValues([HeadCell(y) for y in list(line.keys())])
+        for col in cols:
+            if cols_line.contains(col, case=True, iqual=True):
+                self.elements[col].append(line[col])
 
     def clear(self) -> None:
         for _k in self.elements.keys():
@@ -101,7 +111,7 @@ class SearchableTextPdf(object):
         ou 'nas' em caso de erro nas = Not a String
         """
         try:
-            return ' '.join(self.elements[ColumnsTable.TEXT.value])
+            return ' '.join(self.elements[ColumnsTable.TEXT])
         except Exception as e:
             print(e)
             return 'nas'
@@ -116,7 +126,7 @@ class SearchableTextPdf(object):
 
     def to_file_excel(self, file: File):
         """Exporta os dados da busca para um arquivo .XLSX"""
-        save_data(self.to_data_frame(), file=file)
+        self.to_data_frame().to_excel(file.absolute(), index=False)
 
 
 class ABCDocument(ABC):
@@ -126,10 +136,14 @@ class ABCDocument(ABC):
     - exportar páginas ou arquivos pdf
     - permitir a leitura de arquivos ou bytes de pdf.
     """
-    def __init__(self, mod_doc_pdf: ModDocPdf, name: str):
+    def __init__(self, mod_doc_pdf: ModDocPdf):
         self.mod_doc_pdf: ModDocPdf = mod_doc_pdf
         self.lib_pdf: LibPDF = DEFAULT_LIB_PDF
-        self.name: str = name
+        self.metadata: MetaDataFile = MetaDataFile()
+
+    @property
+    def name(self) -> str | None:
+        return self.metadata.name if not self.metadata.name.is_empty else None
 
     @abstractmethod
     def lenght(self) -> int:
@@ -180,8 +194,8 @@ class ImplementDocumentFitz(ABCDocument):
     """
         Implementação usando a biblioteca fitz.
     """
-    def __init__(self, mod_doc_pdf: ModDocPdf, name: str):
-        super().__init__(mod_doc_pdf, name)
+    def __init__(self, mod_doc_pdf: ModDocPdf):
+        super().__init__(mod_doc_pdf)
         self.mod_doc_pdf: ModDocPdf = mod_doc_pdf
         self.lib_pdf: LibPDF = LibPDF.FITZ
 
@@ -201,6 +215,8 @@ class ImplementDocumentFitz(ABCDocument):
             pg: fitz.Page = self.mod_doc_pdf.load_page(idx)  # retorna fitz.Page
             if pg is not None:
                 _page_pdf = PageDocumentPdf.create_from_page_fitz(pg, pg.number + 1)
+        _page_pdf.metadata = self.metadata
+        _page_pdf.metadata.md5 = MetaDataItem('nan')
         return _page_pdf
 
     def add_page(self, page: PageDocumentPdf):
@@ -231,25 +247,37 @@ class ImplementDocumentFitz(ABCDocument):
         pages = []
         for num, pg in enumerate(self.mod_doc_pdf):
             page_pdf = PageDocumentPdf.create_from_page_fitz(pg, num + 1)
+            page_pdf.metadata = self.metadata
+            page_pdf.metadata.md5 = MetaDataItem('nan')
             pages.append(page_pdf)
         return pages
 
     @classmethod
     def create_from_bytes(cls, bt: BytesIO) -> ImplementDocumentFitz:
         # Cria documento a partir de BytesIO
-        doc: ModDocPdf = fitz.open(stream=bt.getvalue(), filetype="pdf")
+        if not MOD_FITZ:
+            raise ImportError("Módulo fitz|pymupdf não instalado!\nUse pip install fitz.")
+        doc: ModDocPdf = fitz.Document(stream=bt.getvalue(), filetype="pdf")
         name = get_hash_from_bytes(bt)
-        return cls(doc, name)
+        _obj_doc = cls(doc)
+        _obj_doc.metadata.name = MetaDataItem(name)
+        _obj_doc.metadata.md5 = MetaDataItem(name)
+        return _obj_doc
 
     @classmethod
     def create_from_file(cls, file: File) -> ImplementDocumentFitz:
         # Cria documento a partir de caminho no disco
+        if not MOD_FITZ:
+            raise ImportError("Módulo fitz|pymupdf não instalado!\nUse pip install fitz.")
         doc = fitz.open(file.path)
-        return cls(doc, file.name())
+        _obj_doc = cls(doc)
+        _obj_doc.metadata = MetaDataFile.create_metadata(file)
+        return _obj_doc
 
     @classmethod
     def create_from_pages(cls, pages: list[PageDocumentPdf]) -> ImplementDocumentFitz:
-        pdf_document = fitz.Document()  # Criar novo documento PDF.
+        # Criar documento PDF.
+        pdf_document = fitz.Document()
         for page in pages:
             # Verifica se a página é do tipo fitz.Page
             if not isinstance(page.implement_page_pdf.mod_page, fitz.Page):
@@ -269,8 +297,8 @@ class ImplementDocumentPyPdf(ABCDocument):
         Implementação usando a biblioteca pypdf
     """
 
-    def __init__(self, mod_doc_pdf: PdfWriter, name: str):
-        super().__init__(mod_doc_pdf, name)
+    def __init__(self, mod_doc_pdf: PdfWriter):
+        super().__init__(mod_doc_pdf)
         self.mod_doc_pdf: PdfWriter = mod_doc_pdf  # ou PdfReader
         self.lib_pdf: LibPDF = LibPDF.PYPDF
 
@@ -301,7 +329,10 @@ class ImplementDocumentPyPdf(ABCDocument):
             # Acessa a página pela lista self.mod_doc_pdf.pages
             pg = self.mod_doc_pdf.pages[idx]
             # O número da página é idx + 1
-            return PageDocumentPdf.create_from_page_pypdf(pg, idx + 1)
+            page_pdf_doc = PageDocumentPdf.create_from_page_pypdf(pg, idx + 1)
+            page_pdf_doc.metadata = self.metadata
+            page_pdf_doc.metadata.md5 = MetaDataItem('nan')
+            return page_pdf_doc
         return None
 
     def add_page(self, page: PageDocumentPdf):
@@ -325,12 +356,16 @@ class ImplementDocumentPyPdf(ABCDocument):
         pages_pdf: list[PageDocumentPdf] = []
         for num, page in enumerate(self.mod_doc_pdf.pages):
             pg = PageDocumentPdf.create_from_page_pypdf(page, num + 1)
+            pg.metadata = self.metadata
+            pg.metadata.md5 = MetaDataItem('nan')
             pages_pdf.append(pg)
         return pages_pdf
 
     @classmethod
     def create_from_bytes(cls, bt: BytesIO) -> ImplementDocumentPyPdf:
         # Usa PdfReader diretamente de BytesIO
+        if not MOD_PYPDF:
+            raise ImportError("Módulo pypdf não instalado!\nUse pip install pypdf.")
         bt.seek(0)
         reader = PdfReader(bt)
         bt.seek(0)
@@ -341,17 +376,24 @@ class ImplementDocumentPyPdf(ABCDocument):
         bt.close()
         del bt
         del reader
-        return cls(pdf_writer, name)
+        _obj_doc = cls(pdf_writer)
+        _obj_doc.metadata.name = MetaDataItem(name)
+        _obj_doc.metadata.md5 = MetaDataItem(name)
+        return _obj_doc
 
     @classmethod
     def create_from_file(cls, file: File) -> ImplementDocumentPyPdf:
         # Usa PdfReader diretamente de arquivo
+        if not MOD_PYPDF:
+            raise ImportError("Módulo pypdf não instalado!\nUse pip install pypdf.")
         reader = PdfReader(file.absolute())
         pdf_writer = PdfWriter()
         for p in reader.pages:
             pdf_writer.add_page(p)
         del reader
-        return cls(pdf_writer, file.name())
+        _obj_doc = cls(pdf_writer)
+        _obj_doc.metadata = MetaDataFile.create_metadata(file)
+        return _obj_doc
 
     @classmethod
     def create_from_pages(cls, pages: list[PageDocumentPdf]) -> ImplementDocumentPyPdf:
@@ -397,7 +439,9 @@ class DocumentPdf(object):
             _tmp_bytes.seek(0)
             if lib_pdf == LibPDF.PYPDF:
                 name = get_hash_from_bytes(_tmp_bytes)
-                self.doc_pdf = ImplementDocumentPyPdf(document, name)
+                self.doc_pdf = ImplementDocumentPyPdf(document)
+                self.doc_pdf.metadata.name = MetaDataItem(name)
+                self.doc_pdf.metadata.md5 = MetaDataItem(name)
                 del _tmp_bytes
             elif lib_pdf == LibPDF.FITZ:
                 self.doc_pdf = ImplementDocumentFitz.create_from_bytes(_tmp_bytes)
@@ -408,7 +452,9 @@ class DocumentPdf(object):
                 self.doc_pdf = ImplementDocumentPyPdf.create_from_bytes(_tmp_bytes)
             elif lib_pdf == LibPDF.FITZ:
                 name = get_hash_from_bytes(_tmp_bytes)
-                self.doc_pdf = ImplementDocumentFitz(document, name)
+                self.doc_pdf = ImplementDocumentFitz(document)
+                self.doc_pdf.metadata.name = MetaDataItem(name)
+                self.doc_pdf.metadata.md5 = MetaDataItem(name)
                 del _tmp_bytes
         elif isinstance(document, BytesIO):
             if lib_pdf == LibPDF.PYPDF:
@@ -421,8 +467,22 @@ class DocumentPdf(object):
             self.doc_pdf = document
 
         if self.doc_pdf is None:
-            raise ValueError(f'documento inválido, use: str, File, PdfWriter, fitz.Document ou BytesIO não => {type(document)}')
-        self.name: str = self.doc_pdf.name
+            raise ValueError(
+                f'Use: str, File, PdfWriter, fitz.Document ou BytesIO não => {type(document)}'
+            )
+        print(self.metadata)
+
+    @property
+    def name(self) -> str | None:
+        return self.doc_pdf.name
+
+    @property
+    def metadata(self) -> MetaDataFile:
+        return self.doc_pdf.metadata
+
+    @metadata.setter
+    def metadata(self, metadata: MetaDataFile) -> None:
+        self.doc_pdf.metadata = metadata
 
     @property
     def lenght(self) -> int:
@@ -475,28 +535,46 @@ class DocumentPdf(object):
                 show_warning(f'{e}')
         return _values
 
-    def to_dict(self, separator: str = '\n') -> dict[str, list[str]]:
-        dict_doc = {
-            ColumnsTable.NUM_PAGE.value: [],
-            ColumnsTable.NUM_LINHA.value: [],
-            ColumnsTable.TEXT.value: [],
-            ColumnsTable.FILENAME.value: [],
-        }
-        pages = self.to_pages()
-        for page in pages:
-            try:
-                _mp = MapText(page.to_list(separator))
-                _mp.add_column(ColumnsTable.NUM_PAGE.value, [f'{page.number_page}'] * _mp.length)
-                _mp.add_column(ColumnsTable.NUM_PAGE.value, [self.name] * _mp.length)
-            except Exception as e:
-                show_warning(f'{e}')
-            else:
-                dict_doc[ColumnsTable.NUM_PAGE.value].extend(_mp.map_elements[ColumnsTable.NUM_PAGE.value])
-                dict_doc[ColumnsTable.NUM_LINHA.value].extend(_mp.map_elements[ColumnsTable.NUM_LINHA.value])
-                dict_doc[ColumnsTable.TEXT.value].extend(_mp.map_elements[ColumnsTable.TEXT.value])
-                dict_doc[ColumnsTable.FILENAME.value].extend(_mp.map_elements[ColumnsTable.FILENAME.value])
-                del _mp
-        return dict_doc
+    def _create_map(self, page: PageDocumentPdf, *, separator: str = '\n') -> DictTextTable:
+        """
+        @rtype: pd.DataFrame
+        """
+        txt_page = page.to_string()
+        if (txt_page is None) or (txt_page == '') or (txt_page == 'nas'):
+            _values = ['nan']
+        else:
+            _values = txt_page.split(separator)
+
+        if not self.metadata.file_path.is_empty:
+            return DictTextTable.create_from_values(
+                _values,
+                page_num=f'{page.number_page}',
+                file_type='.pdf',
+                file_path=self.metadata.file_path,
+                dir_path=self.metadata.dir_path,
+            )
+        else:
+            return DictTextTable.create_from_values(
+                _values,
+                page_num=f'{page.number_page}',
+                file_type='.pdf',
+                file_path=self.metadata.file_path,
+                dir_path=self.metadata.dir_path,
+            )
+
+    def to_dict(self, separator: str = '\n') -> TableDocuments:
+        list_data: list[DictTextTable] = []
+        pages: list[PageDocumentPdf] = self.to_pages()
+
+        for page_pdf in pages:
+            _mp: DictTextTable = self._create_map(page_pdf, separator=separator)
+            list_data.append(_mp)
+        if len(list_data) == 0:
+            return DictTextTable.create_void_dict()
+        return concat_table_documents(list_data)
+
+    def to_data(self, separator: str = '\n') -> pd.DataFrame:
+        return pd.DataFrame.from_dict(self.to_dict(separator=separator))
 
     def find(
             self, text: str,
@@ -509,26 +587,32 @@ class DocumentPdf(object):
             Filtrar texto retornando a primeira ocorrência do Documento PDF.
         """
         _searchable = SearchableTextPdf(silent)
-        pages_pdf = self.to_pages()
-        for page_pdf in pages_pdf:
-            text_str_in_page = page_pdf.to_string()
+        pages_pdf: list[PageDocumentPdf] = self.to_pages()
+        _page: PageDocumentPdf
+        for _page in pages_pdf:
+            text_str_in_page = _page.to_string()
             if (text_str_in_page == 'nas') or (text_str_in_page is None):
                 continue
             try:
                 fd = FindText(text_str_in_page, separator=separator)
-                idx = fd.index(text, iqual=iqual, case=case)
+                idx = fd.find_index(text, iqual=iqual, case=case)
                 if idx is None:
                     continue
                 math_text = fd.get_index(idx)
-            except:
-                pass
+            except Exception as e:
+                print(f'{__class__.__name__} {e}')
             else:
-                _searchable.add_line(
-                    math_text,
-                    num_page=str(page_pdf.number_page),
-                    num_line=str(idx + 1),
-                    file=self.name,
-                )
+                new_line = {
+                            ColumnsTable.KEY.value: f'{idx}',
+                            ColumnsTable.NUM_PAGE.value: f'{_page.number_page}',
+                            ColumnsTable.NUM_LINE.value: f'{idx+1}',
+                            ColumnsTable.TEXT: math_text,
+                            ColumnsTable.FILE_NAME.value: self.metadata.name,
+                            ColumnsTable.FILETYPE.value: self.metadata.extension,
+                            ColumnsTable.FILE_PATH.value: self.metadata.file_path,
+                            ColumnsTable.DIR.value: self.metadata.dir_path,
+                        }
+                _searchable.add_line(new_line)
                 return _searchable
         return _searchable
 
@@ -545,50 +629,32 @@ class DocumentPdf(object):
         em cada ocorrência.
         """
         _searchable = SearchableTextPdf(silent)
-        _pages_pdf = self.to_pages()
-        for page_pdf in _pages_pdf:
-            text_str_in_page = page_pdf.to_string()
+        pages_pdf: list[PageDocumentPdf] = self.to_pages()
+        _page: PageDocumentPdf
+        for _page in pages_pdf:
+            text_str_in_page = _page.to_string()
             if (text_str_in_page == 'nas') or (text_str_in_page is None):
                 continue
             try:
-                _values = text_str_in_page.split(separator)
-                for num, item in enumerate(_values):
-                    if case:
-                        if iqual:
-                            if text == item:
-                                _searchable.add_line(
-                                    item,
-                                    num_page=str(page_pdf.number_page),
-                                    num_line=str(num + 1),
-                                    file=self.name,
-                                )
-                        else:
-                            if text in item:
-                                _searchable.add_line(
-                                    item,
-                                    num_page=str(page_pdf.number_page),
-                                    num_line=str(num + 1),
-                                    file=self.name,
-                                )
-                    else:
-                        if iqual:
-                            if text.lower() == item.lower():
-                                _searchable.add_line(
-                                    item,
-                                    num_page=str(page_pdf.number_page),
-                                    num_line=str(num + 1),
-                                    file=self.name,
-                                )
-                        else:
-                            if text.lower() in item.lower():
-                                _searchable.add_line(
-                                    item,
-                                    num_page=str(page_pdf.number_page),
-                                    num_line=str(num + 1),
-                                    file=self.name,
-                                )
+                fd = FindText(text_str_in_page, separator=separator)
+                idx = fd.find_index(text, iqual=iqual, case=case)
+                if idx is None:
+                    continue
+                math_text = fd.get_index(idx)
             except Exception as e:
-                show_warning(f'{__class__.__name__} {e}')
+                print(f'{__class__.__name__} {e}')
+            else:
+                new_line = {
+                    ColumnsTable.KEY.value: f'{idx}',
+                    ColumnsTable.NUM_PAGE.value: f'{_page.number_page}',
+                    ColumnsTable.NUM_LINE.value: f'{idx + 1}',
+                    ColumnsTable.TEXT: math_text,
+                    ColumnsTable.FILE_NAME.value: self.metadata.name,
+                    ColumnsTable.FILETYPE.value: self.metadata.extension,
+                    ColumnsTable.FILE_PATH.value: self.metadata.file_path,
+                    ColumnsTable.DIR.value: self.metadata.dir_path,
+                }
+                _searchable.add_line(new_line)
         return _searchable
 
     @classmethod
@@ -607,6 +673,7 @@ class DocumentPdf(object):
             ) -> DocumentPdf:
         if lib_pdf == LibPDF.FITZ:
             mod_pdf = ImplementDocumentFitz.create_from_pages(pages)
+            mod_pdf.metadata.file_path = pages[0].metadata.file_path
             return cls(mod_pdf, lib_pdf=lib_pdf)
         elif lib_pdf == LibPDF.PYPDF:
             mod_pdf = ImplementDocumentPyPdf.create_from_pages(pages)
@@ -615,60 +682,49 @@ class DocumentPdf(object):
             raise NotImplementedError(f'Módulo PDF não implementado: {lib_pdf}')
 
 
-class CollectionPagePdf(object):
+class CollectionPagePdf(ListItems):
     """
         Gerir uma coleção de páginas PDF.
     """
-    def __init__(self, pages: list[PageDocumentPdf] = list()) -> None:
-        self.pages: list[PageDocumentPdf] = pages
+    def __init__(self, pages: list[PageDocumentPdf]) -> None:
+        super().__init__(pages)
         self.pbar: ProgressBarAdapter = ProgressBarAdapter()
 
     @property
     def name(self) -> str | None:
-        if self.is_null:
+        if self.is_empty:
             return None
-        return self.pages[0].__hash__()
-
-    @property
-    def length(self) -> int:
-        return len(self.pages)
-
-    @property
-    def is_null(self) -> bool:
-        return self.length == 0
-
-    def clear(self):
-        self.pages.clear()
+        return self[0].__hash__()
 
     def set_pbar(self, p: ProgressBarAdapter):
         self.pbar = p
 
     def set_land_scape(self):
-        for page in self.pages:
+        for page in self:
             page.set_land_scape()
 
     def set_rotation(self, rotation: int) -> None:
-        for page in self.pages:
+        for page in self:
             page.set_rotation(rotation)
 
     def add_page(self, page: PageDocumentPdf) -> None:
-        page.number_page = len(self.pages) + 1
+        page.number_page = self.length + 1
         self.pbar.update_text(f'Adicionando página PDF {page.number_page}')
-        self.pages.append(page)
+        self.append(page)
 
     def add_pages(self, pages: list[PageDocumentPdf]) -> None:
         _counter = len(pages)
         for n, pg in enumerate(pages):
             _counter += 1
             pg.number_page = _counter
-            self.pages.append(pg)
+            self.append(pg)
 
-    def add_file_pdf(self, file: File, *, lib_pdf: LibPDF = DEFAULT_LIB_PDF) -> None:
+    def add_file_pdf(self, file: File, *, lib_pdf: LibPDF = LibPDF.FITZ) -> None:
         self.pbar.update_text(f'Adicionando arquivo PDF {file.basename()}')
         doc_pdf = DocumentPdf(file, lib_pdf=lib_pdf)
         self.add_pages(doc_pdf.to_pages())
 
-    def add_files_pdf(self, files: list[File], lib_pdf: LibPDF = DEFAULT_LIB_PDF) -> None:
+    def add_files_pdf(self, files: list[File], lib_pdf: LibPDF = LibPDF.FITZ) -> None:
         max_num: int = len(files)
         for num, f in enumerate(files):
             self.pbar.update(
@@ -686,19 +742,15 @@ class CollectionPagePdf(object):
         input_files = InputFiles(d, maxFiles=max_files)
         self.add_files_pdf(input_files.get_files(file_type=LibraryDocs.PDF))
 
-    def set_land_scape(self):
-        for p in self.pages:
-            p.set_land_scape()
-            
     def to_document(self, lib_pdf: LibPDF = DEFAULT_LIB_PDF) -> DocumentPdf:
-        return DocumentPdf.create_from_pages(self.pages, lib_pdf=lib_pdf)
+        return DocumentPdf.create_from_pages(self, lib_pdf=lib_pdf)
 
     def to_file_pdf(self, file: File, *, replace: bool = False) -> None:
         if not replace:
             if file.exists():
                 print(f'[PULANDO]: o arquivo já existe {file.absolute()}')
                 return
-        _doc = DocumentPdf.create_from_pages(self.pages)
+        _doc = DocumentPdf.create_from_pages(self)
         _doc.to_file(file)
         del _doc
 
@@ -711,7 +763,7 @@ class CollectionPagePdf(object):
         max_num = self.length
         self.pbar.start()
         print()
-        for n, page in enumerate(self.pages):
+        for n, page in enumerate(self):
             _doc_pdf = DocumentPdf.create_from_pages([page])
             if prefix is None:
                 file_path = output_dir.join_file(f'{page.number_page}-{_doc_pdf.name}.pdf')
@@ -731,5 +783,3 @@ class CollectionPagePdf(object):
             del _doc_pdf
         print()
         self.pbar.stop()
-
-
